@@ -22,6 +22,8 @@ import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
 
 import java.net.URI;
+import java.util.LinkedHashMap;
+import java.util.Map;
 
 import javax.jms.BytesMessage;
 import javax.jms.Connection;
@@ -36,6 +38,11 @@ import javax.jms.TextMessage;
 import org.apache.activemq.ActiveMQConnectionFactory;
 import org.apache.activemq.broker.BrokerService;
 import org.apache.activemq.broker.TransportConnector;
+import org.apache.activemq.transport.amqp.client.AmqpClient;
+import org.apache.activemq.transport.amqp.client.AmqpConnection;
+import org.apache.activemq.transport.amqp.client.AmqpMessage;
+import org.apache.activemq.transport.amqp.client.AmqpSender;
+import org.apache.activemq.transport.amqp.client.AmqpSession;
 import org.junit.After;
 import org.junit.Test;
 import org.slf4j.Logger;
@@ -188,6 +195,54 @@ public class AmqpTransformerTest {
         openwireConn.close();
     }
 
+    @Test(timeout = 60000)
+    public void testSendAMQPMessageWithComplexAnnotationsReceiveCore() throws Exception {
+        startBrokerWithAmqpTransport(String.format(AMQP_URL, "?transport.transformer=jms"));
+
+        URI remoteURI = new URI("tcp://" + amqpConnectionURI.getHost() + ":" + amqpConnectionURI.getPort());
+        AmqpClient client = new AmqpClient(remoteURI, null, null);
+        AmqpConnection connection = client.connect();
+        try {
+            connection.connect();
+
+            String annotation = "x-opt-embedded-map";
+            Map<String, String> embeddedMap = new LinkedHashMap<>();
+            embeddedMap.put("test-key-1", "value-1");
+            embeddedMap.put("test-key-2", "value-2");
+            embeddedMap.put("test-key-3", "value-3");
+
+            AmqpSession session = connection.createSession();
+            AmqpSender sender = session.createSender(TEST_QUEUE);
+            AmqpMessage message = createAmqpMessage((byte) 'A', 65535);
+
+            message.setApplicationProperty("IntProperty", 42);
+            message.setDurable(true);
+            message.setMessageAnnotation(annotation, embeddedMap);
+            sender.send(message);
+
+            session.close();
+
+            ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(openwireConnectionURI);
+            Connection connection2 = factory.createConnection();
+            try {
+
+                Session session2 = connection2.createSession(false, Session.AUTO_ACKNOWLEDGE);
+                connection2.start();
+                MessageConsumer consumer = session2.createConsumer(session2.createQueue(TEST_QUEUE));
+
+                Message received = consumer.receive(5000);
+                assertNotNull(received);
+                assertEquals(42, received.getIntProperty("IntProperty"));
+
+                connection2.close();
+            } finally {
+                connection2.close();
+            }
+        } finally {
+            connection.close();
+        }
+    }
+
     public void startBrokerWithAmqpTransport(String amqpUrl) throws Exception {
         brokerService = new BrokerService();
         brokerService.setPersistent(false);
@@ -211,4 +266,14 @@ public class AmqpTransformerTest {
             brokerService = null;
         }
     }
+
+    private AmqpMessage createAmqpMessage(byte value, int payloadSize) {
+        AmqpMessage message = new AmqpMessage();
+        byte[] payload = new byte[payloadSize];
+        for (int i = 0; i < payload.length; i++) {
+           payload[i] = value;
+        }
+        message.setBytes(payload);
+        return message;
+     }
 }
