@@ -19,7 +19,9 @@ package org.apache.activemq.broker;
 import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.ArrayList;
 import java.util.LinkedList;
+import java.util.List;
 import java.util.StringTokenizer;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.regex.Pattern;
@@ -74,7 +76,8 @@ public class TransportConnector implements Connector, BrokerServiceAware {
     private int maximumProducersAllowedPerConnection = Integer.MAX_VALUE;
     private int maximumConsumersAllowedPerConnection  = Integer.MAX_VALUE;
     private PublishedAddressPolicy publishedAddressPolicy = new PublishedAddressPolicy();
-    private boolean allowLinkStealing;
+    private boolean allowLinkStealing = false;
+    private boolean warnOnRemoteClose = false;
 
     LinkedList<String> peerBrokers = new LinkedList<String>();
 
@@ -123,7 +126,8 @@ public class TransportConnector implements Connector, BrokerServiceAware {
         rc.setMaximumConsumersAllowedPerConnection(getMaximumConsumersAllowedPerConnection());
         rc.setMaximumProducersAllowedPerConnection(getMaximumProducersAllowedPerConnection());
         rc.setPublishedAddressPolicy(getPublishedAddressPolicy());
-        rc.setAllowLinkStealing(isAllowLinkStealing());
+        rc.setAllowLinkStealing(allowLinkStealing);
+        rc.setWarnOnRemoteClose(isWarnOnRemoteClose());
         return rc;
     }
 
@@ -208,6 +212,7 @@ public class TransportConnector implements Connector, BrokerServiceAware {
         getServer().setAcceptListener(new TransportAcceptListener() {
             @Override
             public void onAccept(final Transport transport) {
+                final String remoteHost = transport.getRemoteAddress();
                 try {
                     brokerService.getTaskRunnerFactory().execute(new Runnable() {
                         @Override
@@ -220,14 +225,12 @@ public class TransportConnector implements Connector, BrokerServiceAware {
                                     throw new BrokerStoppedException("Broker " + brokerService + " is being stopped");
                                 }
                             } catch (Exception e) {
-                                String remoteHost = transport.getRemoteAddress();
                                 ServiceSupport.dispose(transport);
                                 onAcceptError(e, remoteHost);
                             }
                         }
                     });
                 } catch (Exception e) {
-                    String remoteHost = transport.getRemoteAddress();
                     ServiceSupport.dispose(transport);
                     onAcceptError(e, remoteHost);
                 }
@@ -240,9 +243,9 @@ public class TransportConnector implements Connector, BrokerServiceAware {
 
             private void onAcceptError(Exception error, String remoteHost) {
                 if (brokerService != null && brokerService.isStopping()) {
-                    LOG.info("Could not accept connection during shutdown {} : {}", (remoteHost == null ? "" : "from " + remoteHost), error.getLocalizedMessage());
+                    LOG.info("Could not accept connection during shutdown {} : {} ({})", (remoteHost == null ? "" : "from " + remoteHost), error.getLocalizedMessage(), getRootCause(error).getMessage());
                 } else {
-                    LOG.warn("Could not accept connection {} : {}", (remoteHost == null ? "" : "from " + remoteHost), error.getLocalizedMessage());
+                    LOG.warn("Could not accept connection {}: {} ({})", (remoteHost == null ? "" : "from " + remoteHost), error.getMessage(), getRootCause(error).getMessage());
                     LOG.debug("Reason: " + error.getMessage(), error);
                 }
             }
@@ -263,6 +266,20 @@ public class TransportConnector implements Connector, BrokerServiceAware {
         LOG.info("Connector {} started", getName());
     }
 
+    static Throwable getRootCause(final Throwable throwable) {
+        final List<Throwable> list = getThrowableList(throwable);
+        return list.isEmpty() ? null : list.get(list.size() - 1);
+    }
+
+    static List<Throwable> getThrowableList(Throwable throwable) {
+        final List<Throwable> list = new ArrayList<>();
+        while (throwable != null && !list.contains(throwable)) {
+            list.add(throwable);
+            throwable = throwable.getCause();
+        }
+        return list;
+    }
+
     public String getPublishableConnectString() throws Exception {
         String publishableConnectString = publishedAddressPolicy.getPublishableConnectString(this);
         LOG.debug("Publishing: {} for broker transport URI: {}", publishableConnectString, getConnectUri());
@@ -455,8 +472,9 @@ public class TransportConnector implements Connector, BrokerServiceAware {
         if (filter != null) {
             filter = filter.trim();
             if (filter.length() > 0) {
+                result = false;
                 StringTokenizer tokenizer = new StringTokenizer(filter, ",");
-                while (result && tokenizer.hasMoreTokens()) {
+                while (!result && tokenizer.hasMoreTokens()) {
                     String token = tokenizer.nextToken();
                     result = isMatchesClusterFilter(brokerName, token);
                 }
@@ -467,7 +485,7 @@ public class TransportConnector implements Connector, BrokerServiceAware {
     }
 
     private boolean isMatchesClusterFilter(String brokerName, String match) {
-        boolean result = true;
+        boolean result = false;
         if (brokerName != null && match != null && brokerName.length() > 0 && match.length() > 0) {
             result = Pattern.matches(match, brokerName);
         }
@@ -586,8 +604,8 @@ public class TransportConnector implements Connector, BrokerServiceAware {
         return server.isAllowLinkStealing();
     }
 
-    public void setAllowLinkStealing (boolean allowLinkStealing) {
-        this.allowLinkStealing=allowLinkStealing;
+    public void setAllowLinkStealing(boolean allowLinkStealing) {
+        this.allowLinkStealing = allowLinkStealing;
     }
 
     public boolean isAuditNetworkProducers() {
@@ -638,4 +656,12 @@ public class TransportConnector implements Connector, BrokerServiceAware {
     public void setPublishedAddressPolicy(PublishedAddressPolicy publishedAddressPolicy) {
         this.publishedAddressPolicy = publishedAddressPolicy;
     }
+
+    public boolean isWarnOnRemoteClose() {
+        return warnOnRemoteClose;
+    }
+
+    public void setWarnOnRemoteClose(boolean warnOnRemoteClose) {
+        this.warnOnRemoteClose = warnOnRemoteClose;
+    }
 }
