@@ -110,6 +110,8 @@ public class PolicyEntry extends DestinationMapEntry {
      */
     private int optimizeMessageStoreInFlightLimit = 10;
     private boolean persistJMSRedelivered = false;
+    private int sendFailIfNoSpace = -1;
+    private long sendFailIfNoSpaceAfterTimeout = -1;
 
 
     public void configure(Broker broker,Queue queue) {
@@ -148,7 +150,7 @@ public class PolicyEntry extends DestinationMapEntry {
      *
      * If includedProperties is null then all of the properties will be set as
      * isUpdate will return true
-     * @param baseDestination
+     * @param queue
      * @param includedProperties
      */
     public void update(Queue queue, Set<String> includedProperties) {
@@ -310,6 +312,12 @@ public class PolicyEntry extends DestinationMapEntry {
         }
         destination.setSlowConsumerStrategy(scs);
         destination.setPrioritizedMessages(isPrioritizedMessages());
+        if (sendFailIfNoSpace != -1) {
+            destination.getSystemUsage().setSendFailIfNoSpace(isSendFailIfNoSpace());
+        }
+        if (sendFailIfNoSpaceAfterTimeout != -1) {
+            destination.getSystemUsage().setSendFailIfNoSpaceAfterTimeout(getSendFailIfNoSpaceAfterTimeout());
+        }
     }
 
     public void configure(Broker broker, SystemUsage memoryManager, TopicSubscription subscription) {
@@ -1029,7 +1037,7 @@ public class PolicyEntry extends DestinationMapEntry {
     /**
      * Sets the amount of time a destination is inactive before it is marked for GC
      *
-     * @param inactiveTimoutBeforeGC
+     * @param inactiveTimeoutBeforeGC
      *        time in milliseconds to configure as the inactive timeout.
      */
     public void setInactiveTimeoutBeforeGC(long inactiveTimeoutBeforeGC) {
@@ -1103,6 +1111,26 @@ public class PolicyEntry extends DestinationMapEntry {
         return "PolicyEntry [" + destination + "]";
     }
 
+    public void setSendFailIfNoSpace(boolean val) {
+        if (val) {
+            this.sendFailIfNoSpace = 1;
+        } else {
+            this.sendFailIfNoSpace = 0;
+        }
+    }
+
+    public boolean isSendFailIfNoSpace() {
+        return sendFailIfNoSpace == 1;
+    }
+
+    public void setSendFailIfNoSpaceAfterTimeout(long sendFailIfNoSpaceAfterTimeout) {
+        this.sendFailIfNoSpaceAfterTimeout = sendFailIfNoSpaceAfterTimeout;
+    }
+
+    public long getSendFailIfNoSpaceAfterTimeout() {
+        return this.sendFailIfNoSpaceAfterTimeout;
+    }
+
     public boolean isUseTopicSubscriptionInflightStats() {
         return useTopicSubscriptionInflightStats;
     }
