@@ -19,6 +19,8 @@ package org.apache.activemq.broker.scheduler;
 import java.io.IOException;
 import java.util.concurrent.atomic.AtomicBoolean;
 
+import javax.jms.MessageFormatException;
+
 import org.apache.activemq.ScheduledMessage;
 import org.apache.activemq.advisory.AdvisorySupport;
 import org.apache.activemq.broker.Broker;
@@ -55,6 +57,11 @@ import org.slf4j.LoggerFactory;
 public class SchedulerBroker extends BrokerFilter implements JobListener {
     private static final Logger LOG = LoggerFactory.getLogger(SchedulerBroker.class);
     private static final IdGenerator ID_GENERATOR = new IdGenerator();
+    private static final LongSequenceGenerator longGenerator = new LongSequenceGenerator();
+    /**
+     * The max repeat value allowed to prevent clients from causing DoS issues with huge repeat counts
+     */
+    private static final int MAX_REPEAT_ALLOWED = 1000;
     private final LongSequenceGenerator messageIdGenerator = new LongSequenceGenerator();
     private final AtomicBoolean started = new AtomicBoolean();
     private final WireFormat wireFormat = new OpenWireFormat();
@@ -64,6 +71,7 @@ public class SchedulerBroker extends BrokerFilter implements JobListener {
 
     private final JobSchedulerStore store;
     private JobScheduler scheduler;
+    private int maxRepeatAllowed = MAX_REPEAT_ALLOWED;
 
     public SchedulerBroker(BrokerService brokerService, Broker next, JobSchedulerStore store) throws Exception {
         super(next);
@@ -335,10 +343,16 @@ public class SchedulerBroker extends BrokerFilter implements JobListener {
         Object repeatValue = msg.getProperty(ScheduledMessage.AMQ_SCHEDULED_REPEAT);
         if (repeatValue != null) {
             repeat = (Integer) TypeConversionSupport.convert(repeatValue, Integer.class);
+            if (repeat > maxRepeatAllowed) {
+                throw new MessageFormatException("The scheduled repeat value is too large");
+            }
         }
 
-        getInternalScheduler().schedule(msg.getMessageId().toString(),
-            new ByteSequence(packet.data, packet.offset, packet.length), cronEntry, delay, period, repeat);
+        //job id should be unique for every job (Same format as MessageId)
+        MessageId jobId = new MessageId(messageSend.getMessageId().getProducerId(), longGenerator.getNextSequenceId());
+
+        getInternalScheduler().schedule(jobId.toString(),
+                new ByteSequence(packet.data, packet.offset, packet.length), cronEntry, delay, period, repeat);
     }
 
     @Override
@@ -353,6 +367,9 @@ public class SchedulerBroker extends BrokerFilter implements JobListener {
             int repeat = 0;
             if (repeatValue != null) {
                 repeat = (Integer) TypeConversionSupport.convert(repeatValue, Integer.class);
+                if (repeat > maxRepeatAllowed) {
+                    throw new MessageFormatException("The scheduled repeat value is too large");
+                }
             }
 
             if (repeat != 0 || cronStr != null && cronStr.length() > 0) {
@@ -452,4 +469,12 @@ public class SchedulerBroker extends BrokerFilter implements JobListener {
             LOG.error("Failed to send scheduled message {}", job.getJobId(), e);
         }
     }
+
+    public int getMaxRepeatAllowed() {
+        return maxRepeatAllowed;
+    }
+
+    public void setMaxRepeatAllowed(int maxRepeatAllowed) {
+        this.maxRepeatAllowed = maxRepeatAllowed;
+    }
 }
