@@ -111,6 +111,10 @@ public class TransactionContext implements XAResource {
         rollbackOnly = val;
     }
 
+    public boolean isRollbackOnly() {
+        return rollbackOnly;
+    }
+
     public boolean isInLocalTransaction() {
         return transactionId != null && transactionId.isLocalTransaction();
     }
@@ -277,7 +281,7 @@ public class TransactionContext implements XAResource {
             TransactionInfo info = new TransactionInfo(getConnectionId(), transactionId, TransactionInfo.ROLLBACK);
             this.transactionId = null;
             //make this synchronous - see https://issues.apache.org/activemq/browse/AMQ-2364
-            this.connection.syncSendPacket(info);
+            this.connection.syncSendPacket(info, this.connection.isClosing() ? this.connection.getCloseTimeout() : 0);
             // Notify the listener that the tx was rolled back
             if (localTransactionEventListener != null) {
                 localTransactionEventListener.rollbackEvent();
@@ -545,7 +549,11 @@ public class TransactionContext implements XAResource {
             // No risk for concurrent updates as we own the list now
             if (l != null) {
                 for (TransactionContext ctx : l) {
-                    ctx.afterRollback();
+                    try {
+                        ctx.afterRollback();
+                    } catch (Exception ignored) {
+                        LOG.debug("ignoring exception from after rollback on ended transaction: {}", ignored, ignored);
+                    }
                 }                  
             }
         } catch (JMSException e) {
@@ -672,26 +680,32 @@ public class TransactionContext implements XAResource {
     @Override
     public Xid[] recover(int flag) throws XAException {
         LOG.debug("recover({})", flag);
+        XATransactionId[] answer;
 
-        TransactionInfo info = new TransactionInfo(getConnectionId(), null, TransactionInfo.RECOVER);
-        try {
-            this.connection.checkClosedOrFailed();
-            this.connection.ensureConnectionInfoSent();
+        if (XAResource.TMNOFLAGS == flag) {
+            // signal next in cursor scan, which for us is always the end b/c we don't maintain any cursor state
+            // allows looping scan to complete
+            answer = new XATransactionId[0];
+        } else {
+            TransactionInfo info = new TransactionInfo(getConnectionId(), null, TransactionInfo.RECOVER);
+            try {
+                this.connection.checkClosedOrFailed();
+                this.connection.ensureConnectionInfoSent();
 
-            DataArrayResponse receipt = (DataArrayResponse)this.connection.syncSendPacket(info);
-            DataStructure[] data = receipt.getData();
-            XATransactionId[] answer;
-            if (data instanceof XATransactionId[]) {
-                answer = (XATransactionId[])data;
-            } else {
-                answer = new XATransactionId[data.length];
-                System.arraycopy(data, 0, answer, 0, data.length);
+                DataArrayResponse receipt = (DataArrayResponse) this.connection.syncSendPacket(info);
+                DataStructure[] data = receipt.getData();
+                if (data instanceof XATransactionId[]) {
+                    answer = (XATransactionId[]) data;
+                } else {
+                    answer = new XATransactionId[data.length];
+                    System.arraycopy(data, 0, answer, 0, data.length);
+                }
+            } catch (JMSException e) {
+                throw toXAException(e);
             }
-            LOG.debug("recover({})={}", flag, answer);
-            return answer;
-        } catch (JMSException e) {
-            throw toXAException(e);
         }
+        LOG.debug("recover({})={}", flag, answer);
+        return answer;
     }
 
     @Override
@@ -781,16 +795,20 @@ public class TransactionContext implements XAResource {
      * @param e JMSException to convert
      * @return XAException wrapping original exception or its message
      */
-    private XAException toXAException(JMSException e) {
+    public static XAException toXAException(JMSException e) {
         if (e.getCause() != null && e.getCause() instanceof XAException) {
             XAException original = (XAException)e.getCause();
             XAException xae = new XAException(original.getMessage());
-            xae.errorCode = original.errorCode;
-            if (xae.errorCode == XA_OK) {
+            if (original != null) {
+                xae.errorCode = original.errorCode;
+            }
+            if (original != null && xae != null && xae.errorCode == XA_OK) {
                 // detail not unmarshalled see: org.apache.activemq.openwire.v1.BaseDataStreamMarshaller.createThrowable
                 xae.errorCode = parseFromMessageOr(original.getMessage(), XAException.XAER_RMERR);
             }
-            xae.initCause(original);
+            if (original != null) {
+                xae.initCause(original);
+            }
             return xae;
         }
 
@@ -800,7 +818,7 @@ public class TransactionContext implements XAResource {
         return xae;
     }
 
-    private int parseFromMessageOr(String message, int fallbackCode) {
+    private static int parseFromMessageOr(String message, int fallbackCode) {
         final String marker = "xaErrorCode:";
         final int index = message.lastIndexOf(marker);
         if (index > -1) {
