@@ -172,9 +172,9 @@ public class MultiKahaDBPersistenceAdapter extends LockableServiceSupport implem
     }
 
     @Override
-    public void checkpoint(final boolean sync) throws IOException {
+    public void checkpoint(final boolean cleanup) throws IOException {
         for (PersistenceAdapter persistenceAdapter : adapters) {
-            persistenceAdapter.checkpoint(sync);
+            persistenceAdapter.checkpoint(cleanup);
         }
     }
 
@@ -198,11 +198,11 @@ public class MultiKahaDBPersistenceAdapter extends LockableServiceSupport implem
         if (filteredAdapter.getDestination() == matchAll && filteredAdapter.isPerDestination()) {
             filteredAdapter = addAdapter(filteredAdapter, destination);
             if (LOG.isTraceEnabled()) {
-                LOG.info("created per destination adapter for: " + destination  + ", " + result);
+                LOG.trace("created per destination adapter for: " + destination  + ", " + result);
             }
         }
         startAdapter(filteredAdapter.getPersistenceAdapter(), destination.getQualifiedName());
-        LOG.debug("destination {} matched persistence adapter {}", new Object[]{destination.getQualifiedName(), filteredAdapter.getPersistenceAdapter()});
+        LOG.debug("destination {} matched persistence adapter {}", destination.getQualifiedName(), filteredAdapter.getPersistenceAdapter());
         return filteredAdapter.getPersistenceAdapter();
     }
 
@@ -330,11 +330,11 @@ public class MultiKahaDBPersistenceAdapter extends LockableServiceSupport implem
         if (adapterDir != null) {
             if (IOHelper.deleteFile(adapterDir)) {
                 if (LOG.isTraceEnabled()) {
-                    LOG.info("deleted per destination adapter directory for: " + destination);
+                    LOG.trace("deleted per destination adapter directory for: " + destination);
                 }
             } else {
                 if (LOG.isTraceEnabled()) {
-                    LOG.info("failed to deleted per destination adapter directory for: " + destination);
+                    LOG.trace("failed to deleted per destination adapter directory for: " + destination);
                 }
             }
         }
@@ -562,6 +562,14 @@ public class MultiKahaDBPersistenceAdapter extends LockableServiceSupport implem
         return transactionStore.getJournalCleanupInterval();
     }
 
+    public void setCheckForCorruption(boolean checkForCorruption) {
+        transactionStore.setCheckForCorruption(checkForCorruption);
+    }
+
+    public boolean isCheckForCorruption() {
+        return transactionStore.isCheckForCorruption();
+    }
+
     public List<PersistenceAdapter> getAdapters() {
         return Collections.unmodifiableList(adapters);
     }
