@@ -25,6 +25,7 @@ import java.lang.reflect.Field;
 import java.net.URI;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.List;
 import java.util.Set;
 import java.util.concurrent.ConcurrentMap;
 
@@ -39,6 +40,7 @@ import org.apache.activemq.advisory.AdvisoryBroker;
 import org.apache.activemq.advisory.AdvisorySupport;
 import org.apache.activemq.broker.BrokerPlugin;
 import org.apache.activemq.broker.BrokerService;
+import org.apache.activemq.broker.TransportConnector;
 import org.apache.activemq.broker.region.DestinationInterceptor;
 import org.apache.activemq.broker.region.DestinationStatistics;
 import org.apache.activemq.broker.region.virtual.CompositeQueue;
@@ -116,31 +118,49 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
      * @throws Exception
      */
     @Test(timeout = 60 * 1000)
-    public void testVirtualTopic() throws Exception {
+    public void testVirtualTopics() throws Exception {
         Assume.assumeTrue(isUseVirtualDestSubsOnCreation);
         doSetUp(true, null);
 
         MessageConsumer advisoryConsumer = getVirtualDestinationAdvisoryConsumer("VirtualTopic.>");
 
         MessageProducer includedProducer = localSession.createProducer(new ActiveMQTopic("VirtualTopic.include.test.bar"));
+        MessageProducer includedProducer2 = localSession.createProducer(new ActiveMQTopic("VirtualTopic.include.test.bar2"));
+        MessageProducer includedProducer3 = localSession.createProducer(new ActiveMQTopic("VirtualTopic.include.test.bar3"));
         Thread.sleep(2000);
         Message test = localSession.createTextMessage("test");
 
         final DestinationStatistics destinationStatistics = localBroker.getDestination(new ActiveMQTopic("VirtualTopic.include.test.bar")).getDestinationStatistics();
+        final DestinationStatistics destinationStatistics2 = localBroker.getDestination(new ActiveMQTopic("VirtualTopic.include.test.bar2")).getDestinationStatistics();
+
+        //No queue destination on the remote side so should not forward
+        final DestinationStatistics destinationStatistics3 = localBroker.getDestination(new ActiveMQTopic("VirtualTopic.include.test.bar3")).getDestinationStatistics();
 
         //this will create the destination so messages accumulate
         final DestinationStatistics remoteStats = remoteBroker.getDestination(new ActiveMQQueue("Consumer.cons1.VirtualTopic.include.test.bar")).getDestinationStatistics();
+        final DestinationStatistics remoteStats2 = remoteBroker.getDestination(new ActiveMQQueue("Consumer.cons1.VirtualTopic.include.test.bar2")).getDestinationStatistics();
+
         waitForConsumerCount(destinationStatistics, 1);
+        waitForConsumerCount(destinationStatistics2, 1);
 
         includedProducer.send(test);
+        includedProducer2.send(localSession.createTextMessage("test2"));
+        includedProducer3.send(localSession.createTextMessage("test3"));
 
         //assert statistics
         waitForDispatchFromLocalBroker(destinationStatistics, 1);
+        waitForDispatchFromLocalBroker(destinationStatistics2, 1);
         assertLocalBrokerStatistics(destinationStatistics, 1);
+        assertLocalBrokerStatistics(destinationStatistics2, 1);
         assertEquals("remote dest messages", 1, remoteStats.getMessages().getCount());
+        assertEquals("remote dest messages", 1, remoteStats2.getMessages().getCount());
 
-        assertRemoteAdvisoryCount(advisoryConsumer, 1);
-        assertAdvisoryBrokerCounts(1,1,1);
+        assertRemoteAdvisoryCount(advisoryConsumer, 2);
+        assertAdvisoryBrokerCounts(1,2,2);
+
+        //should not have forwarded for 3rd topic
+        Thread.sleep(1000);
+        assertEquals("local broker dest stat dispatched", 0, destinationStatistics3.getDispatched().getCount());
     }
 
 
@@ -240,9 +260,18 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
      */
     @Test(timeout = 60 * 1000)
     public void testDynamicFlow() throws Exception {
+        testDynamicFlow(false);
+    }
+
+    @Test(timeout = 60 * 1000)
+    public void testDynamicFlowForceDurable() throws Exception {
+        testDynamicFlow(true);
+    }
+
+    protected void testDynamicFlow(boolean forceDurable) throws Exception {
         Assume.assumeTrue(isUseVirtualDestSubsOnCreation);
 
-        doSetUp(true, null);
+        doSetUp(true, null, true, forceDurable);
 
         MessageConsumer advisoryConsumer = getVirtualDestinationAdvisoryConsumer(testTopicName);
 
@@ -262,6 +291,7 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
                 new ActiveMQQueue("include.test.bar.bridge")).getDestinationStatistics();
 
         waitForConsumerCount(destinationStatistics, 1);
+        assertNCDurableSubsCount(localBroker, included, forceDurable ? 1 : 0);
         includedProducer.send(test);
 
         waitForDispatchFromLocalBroker(destinationStatistics, 1);
@@ -272,7 +302,6 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
         assertAdvisoryBrokerCounts(1,1,1);
     }
 
-
     /**
      * Test that dynamic flow works for virtual destinations when a second composite
      * topic is included that forwards to the same queue, but is excluded from
@@ -544,6 +573,59 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
 
     }
 
+    /**
+     * This tests that having 2 composite destinations (1 included for dynamic flow and 1 not)
+     * will allow messages to flow and that deleting 1 destination dosen't clear out the virtual
+     * consumer map except for what should be cleared.
+     *
+     */
+    @Test(timeout = 60 * 1000)
+    public void testTwoCompositeTopicsRemove1() throws Exception {
+        Assume.assumeTrue(isUseVirtualDestSubsOnCreation);
+
+        doSetUp(true, null);
+
+        //configure a virtual destination that forwards messages from topic testQueueName
+        //to queue "include.test.bar.bridge" and "include.test.bar.bridge2"
+        CompositeTopic compositeTopic1 = createCompositeTopic(testTopicName,
+                new ActiveMQQueue("include.test.bar.bridge"));
+        CompositeTopic compositeTopic2 = createCompositeTopic(testTopicName + 2,
+                new ActiveMQQueue("include.test.bar.bridge2"));
+
+        runtimeBroker.setVirtualDestinations(new VirtualDestination[] {compositeTopic1, compositeTopic2}, true);
+
+        MessageProducer includedProducer = localSession.createProducer(included);
+        Message test = localSession.createTextMessage("test");
+        Thread.sleep(1000);
+
+        final DestinationStatistics destinationStatistics = localBroker.getDestination(included).getDestinationStatistics();
+        final DestinationStatistics remoteDestStatistics = remoteBroker.getDestination(
+                new ActiveMQQueue("include.test.bar.bridge")).getDestinationStatistics();
+
+        waitForConsumerCount(destinationStatistics, 1);
+
+        includedProducer.send(test);
+
+        waitForDispatchFromLocalBroker(destinationStatistics, 1);
+        assertLocalBrokerStatistics(destinationStatistics, 1);
+        assertEquals("remote dest messages", 1, remoteDestStatistics.getMessages().getCount());
+
+        //verify there are 2 virtual destinations but only 1 consumer and broker dest
+        assertAdvisoryBrokerCounts(2,1,1);
+        runtimeBroker.setVirtualDestinations(new VirtualDestination[] {compositeTopic1}, true);
+        Thread.sleep(2000);
+        //verify there is is only 1 virtual dest after deletion
+        assertAdvisoryBrokerCounts(1,1,1);
+
+        includedProducer.send(test);
+
+        //make sure messages are still forwarded even after 1 composite topic was deleted
+        waitForDispatchFromLocalBroker(destinationStatistics, 2);
+        assertLocalBrokerStatistics(destinationStatistics, 2);
+        assertEquals("remote dest messages", 2, remoteDestStatistics.getMessages().getCount());
+
+    }
+
     /**
      * Test that demand is destroyed after removing both targets from the composite Topic
      * @throws Exception
@@ -953,7 +1035,7 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
         CompositeTopic compositeTopic = createCompositeTopic(testTopicName,
                 new ActiveMQQueue("include.test.bar.bridge"));
 
-        doSetUp(true, new VirtualDestination[] {compositeTopic}, false);
+        doSetUp(true, new VirtualDestination[] {compositeTopic}, false, false);
 
         MessageConsumer advisoryConsumer = getVirtualDestinationAdvisoryConsumer(testTopicName);
 
@@ -981,7 +1063,7 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
         CompositeTopic compositeTopic = createCompositeTopic(testTopicName,
                 new ActiveMQQueue("include.test.bar.bridge"));
 
-        doSetUp(true, new VirtualDestination[] {compositeTopic}, false);
+        doSetUp(true, new VirtualDestination[] {compositeTopic}, false, false);
 
         MessageConsumer advisoryConsumer = getVirtualDestinationAdvisoryConsumer(testTopicName);
 
@@ -1236,34 +1318,18 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
         doTearDown();
     }
 
-    protected void doTearDown() throws Exception {
-        if (localConnection != null) {
-            localConnection.close();
-        }
-        if (remoteConnection != null) {
-            remoteConnection.close();
-        }
-        if (localBroker != null) {
-            localBroker.stop();
-        }
-        if (remoteBroker != null) {
-            remoteBroker.stop();
-        }
-    }
-
-
     protected void doSetUp(boolean deleteAllMessages,
             VirtualDestination[] remoteVirtualDests) throws Exception {
-        doSetUp(deleteAllMessages, remoteVirtualDests, true);
+        doSetUp(deleteAllMessages, remoteVirtualDests, true, false);
     }
 
     protected void doSetUp(boolean deleteAllMessages,
-            VirtualDestination[] remoteVirtualDests, boolean startNetworkConnector) throws Exception {
+            VirtualDestination[] remoteVirtualDests, boolean startNetworkConnector, boolean forceDurable) throws Exception {
         remoteBroker = createRemoteBroker(isUseVirtualDestSubsOnCreation, remoteVirtualDests);
         remoteBroker.setDeleteAllMessagesOnStartup(deleteAllMessages);
         remoteBroker.start();
         remoteBroker.waitUntilStarted();
-        localBroker = createLocalBroker(startNetworkConnector);
+        localBroker = createLocalBroker(startNetworkConnector, forceDurable);
         localBroker.setDeleteAllMessagesOnStartup(deleteAllMessages);
         localBroker.start();
         localBroker.waitUntilStarted();
@@ -1287,13 +1353,16 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
 
 
     protected NetworkConnector connector;
-    protected BrokerService createLocalBroker(boolean startNetworkConnector) throws Exception {
+    protected BrokerService createLocalBroker(boolean startNetworkConnector, boolean forceDurable) throws Exception {
         BrokerService brokerService = new BrokerService();
         brokerService.setMonitorConnectionSplits(true);
         brokerService.setDataDirectoryFile(tempFolder.newFolder());
         brokerService.setBrokerName("localBroker");
 
-        connector = new DiscoveryNetworkConnector(new URI("static:(tcp://localhost:61617)"));
+        List<TransportConnector> transportConnectors = remoteBroker.getTransportConnectors();
+        URI remoteURI = transportConnectors.get(0).getConnectUri();
+        String uri = "static:(" + remoteURI + ")";
+        connector = new DiscoveryNetworkConnector(new URI(uri));
         connector.setName("networkConnector");
         connector.setDynamicOnly(false);
         connector.setDecreaseNetworkConsumerPriority(false);
@@ -1301,7 +1370,7 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
         connector.setDuplex(isDuplex);
         connector.setUseVirtualDestSubs(true);
         connector.setDynamicallyIncludedDestinations(Lists.newArrayList(new ActiveMQQueue(testQueueName),
-                new ActiveMQTopic(testTopicName), new ActiveMQTopic("VirtualTopic.>")));
+                new ActiveMQTopic(testTopicName + (forceDurable ? "?forceDurable=true" : "")), new ActiveMQTopic("VirtualTopic.>")));
         connector.setExcludedDestinations(Lists.newArrayList(new ActiveMQQueue("exclude.test.foo"),
                 new ActiveMQTopic("exclude.test.bar")));
 
@@ -1309,7 +1378,7 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
             brokerService.addNetworkConnector(connector);
         }
 
-        brokerService.addConnector("tcp://localhost:61616");
+        brokerService.addConnector("tcp://localhost:0");
 
         return brokerService;
     }
@@ -1337,7 +1406,7 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
         remoteAdvisoryBroker = (AdvisoryBroker)
                 brokerService.getBroker().getAdaptor(AdvisoryBroker.class);
 
-        brokerService.addConnector("tcp://localhost:61617");
+        brokerService.addConnector("tcp://localhost:0");
 
         return brokerService;
     }
@@ -1375,7 +1444,7 @@ public class VirtualConsumerDemandTest extends DynamicNetworkTestSupport {
         ActiveMQMessage message = null;
         while ((message = (ActiveMQMessage) advisoryConsumer.receive(1000)) != null) {
             available++;
-            LOG.debug("advisory data structure: {}", message.getDataStructure());
+            LOG.info("advisory data structure: {}", message.getDataStructure());
         }
         assertEquals(count, available);
     }
